from kivy.app import App
from kivy.lang import Builder
from kivy.core.window import Window
from kivy.clock import Clock
from kivy.storage.jsonstore import JsonStore
from kivy.metrics import cm,dp,sp,Metrics
from kivy.uix.screenmanager import ScreenManager, Screen
from kivy.uix.actionbar import ActionBar,ActionPrevious,ActionView,ActionButton
from kivy.uix.label import Label
from functools import partial
from kivy.uix.gridlayout import GridLayout
from kivy.uix.stacklayout import StackLayout
from kivy.uix.floatlayout import FloatLayout
from kivy.uix.relativelayout import RelativeLayout
from kivy.uix.boxlayout import BoxLayout
from kivy.uix.textinput import TextInput
from kivy.uix.button import Button
from kivy.uix.widget import Widget
from kivy.factory import Factory
from kivy.uix.popup import Popup
from kivy.properties import ObjectProperty, StringProperty, ReferenceListProperty, NumericProperty
from kivy.uix.scrollview import ScrollView
from kivy.uix.togglebutton import ToggleButton
from kivy.uix.scatter import Scatter
from gif_builder import gif_build
from filtering import max_filter_3d, min_filter_3d, median_filter_3d
from morphological_3D import remove_small_obj, erosion, skeletonize
from interpolation_3D import interpolate
from fft import fast_fourier_transform
import os
import cv2
from natsort import natsorted
import numpy as np


# Create both screens. Please note the root.manager.current: this is how
# you can control the ScreenManager from kv. Each screen has by default a
# property manager that gives you the instance of the ScreenManager used.
Builder.load_string("""
#:import Factory kivy.factory.Factory

<StartingScreen>:
    canvas.before:
        Color:
            rgba: 0, 0, 1, .5
        Rectangle:
            pos: self.pos
            size: self.size
    FloatLayout:
        size: root.width, root.height
        background_color: (1,1,0,1)
        ActionBar: 
            pos_hint: {'top':1} 
            size: root.width, root.height/14
            ActionView: 
                use_separator: True
                ActionPrevious:
                    size_hint_x:None
                    width:0
                    app_icon_width:.1
                    title:'Action Bar'
                    with_previous:False
                ActionButton:
                    text: 'Help'
                    on_release: Factory.Help().open()

        Label:
            size_hint: .4, .2
            pos_hint: {"x": .3, "y":.7}
            text: '3D Image Processing Software'
            font_size: '20sp'
        Button:
            text:"Filter Selection"
            size_hint: .2, .2
            pos_hint: {"x": 0, "y":.7}
            on_release: Factory.FilterSelection().open()
        Button:
            text:"3D Morphological\\nOperator Selection"
            size_hint: .2, .2
            pos_hint: {"x": 0, "y":.4}
            on_release: Factory.MorphologicalSelection().open()
        Button:
            text:"View Resulted GIF"
            background_color:(1,0,0,1)
            size_hint: .3, .3
            pos_hint: {"x": .35, "y": .05}
            on_release: Factory.GifView().open()
        Button:
            text:"Reset GIF"
            background_color:(1,1,0,1)
            size_hint: .2, .2
            pos_hint: {"x": .4, "y": .35}
            on_release: root.reset_gif()
        Button:
            text:"Interpolation"
            size_hint: .2, .2
            pos_hint: {"x": 0, "y": .1}
            on_release: Factory.Interpolation().open()
        Button:
            text:"Fast-Fourier-Transform"
            background_color:(1,1,1,1)
            size_hint: .2, .2
            pos_hint: {"x": .8, "y": .7}
            on_release: root.fourier()
<Help>:
    title: 'Help:'
    BoxLayout:
        orientation: 'vertical'
        Label:
            id: help_text
            text_size: self.size
            halign: 'center'
            markup:True
            valign: 'middle'

        Button:
            text: "Dismiss"
            size_hint: (1,0.2)
            background_color:(1,0,0,1)
            on_release: root.dismiss()
    
<GifView>:
    title: 'Resulted GIF:'
    BoxLayout:
        orientation: 'vertical'
        Image:
            id: resulted_gif
            source: 'GIFs/result.gif'
            anim_delay: 0.1
            mipmap: True
            allow_stretch: True
            #self.ids.image1.reload()
        Button:
            text: "Dismiss"
            size_hint: (1,0.2)
            background_color:(1,0,0,1)
            on_release: root.dismiss()
<FilterSelection>:
    title: 'Select the filter to apply:'
    size_hint: (.75, .75)
    GridLayout:
        cols: 1

        Button:
            id: median
            text: "1. Median Filter"
            on_release: root.dismiss()
            on_release: root.median_filter()
        Button:
            id: min
            text: "2. MIN Filter"
            on_release: root.dismiss()
            on_release: root.min_filter()
        Button:
            id: max
            text: "3. MAX Filter"
            on_release: root.dismiss()
            on_release: root.max_filter()
        BoxLayout:
            size_hint: (1,.4)
            #cols: 2
            Label:
                #size_hint: (1,.4)
                text: 'Select the size of the filter (5-20 recommended):'
            TextInput:
                id: filter_number
                multiline: False
                halign: 'center'
                input_filter: "int"
                size_hint: (.4,1)
                text: '5'
                on_text: 
                    root.check_text()
        Button:
            text: "Dismiss"
            background_color:(1,0,0,1)
            on_release: root.dismiss()
<MorphologicalSelection>:
    title: 'Select the morphological operator you would like to apply:'
    size_hint: (.75, .75)
    GridLayout:
        cols: 1

        Button:
            id: small_objects
            text: "1. Remove Small Objects"
            on_release: root.dismiss()
            on_release: root.small_objects()
        Button:
            id: skeletonize
            text: "2. Skeletonize 3D"
            on_release: root.dismiss()
            on_release: root.skeletonize()
        Button:
            id: greyscale_erosion
            text: "3. Greyscale Erosion"
            on_release: root.dismiss()
            on_release: root.erosion()
        Button:
            text: "Dismiss"
            background_color:(1,0,0,1)
            on_release: root.dismiss()
<Interpolation>:
    title: 'Interpolate 3D:'
    size_hint: (.75, .75)
    BoxLayout:
        orientation: 'vertical'
        GridLayout:
            size_hint_y: 2
            cols: 2
            Label:
                #size_hint: (1,.4)
                text: 'X:'
            TextInput:
                id: X
                multiline: False
                halign: 'center'
                input_filter: "float"
                size_hint: (.4,1)
                text: '1.0'
                on_text: 
                    root.check_text_x()
            Label:
                #size_hint: (1,.4)
                text: 'Y:'
            TextInput:
                id: Y
                multiline: False
                halign: 'center'
                input_filter: "float"
                size_hint: (.4,1)
                text: '1.0'
                on_text: 
                    root.check_text_y()
            Label:
                #size_hint: (1,.4)
                text: 'Z:'
            TextInput:
                id: Z
                multiline: False
                halign: 'center'
                input_filter: "float"
                size_hint: (.4,1)
                text: '1.0'
                on_text: 
                    root.check_text_z()
        Button:
            size_hint_x: .3
            pos_hint: {'x': .35}
            id: interpolate
            text: "Interpolate"
            on_release: root.dismiss()
            on_release: root.interpolation()
        Button:
            text: "Dismiss"
            background_color:(1,0,0,1)
            on_release: root.dismiss()
""")

PATH = 'sample2_neuron/'
list_files = os.listdir(PATH)
list_files = natsorted(list_files)
image_list = []
for image in list_files:
    img = cv2.imread(PATH + image, cv2.IMREAD_GRAYSCALE)
    image_list.append(img)
filter_type = 0
image_volume = np.stack(image_list, axis=0)

class StartingScreen(Screen):
    def reset_gif(self):
        global image_volume
        os.remove("GIFs/result.gif")
        image_volume = np.stack(image_list, axis=0)

    def fourier(self):
        global image_volume
        image_volume = fast_fourier_transform(image_volume)
        print(image_volume)
        build_gif()

class Help(Popup):
    def on_pre_open(self):
        text = open('help_tab.txt', 'r').read()
        self.ids.help_text.text = text


class Interpolation(Popup):
    def check_text_x(self):
        try:
            self.ids.X.text = str(float(self.ids.X.text))
            if (float(self.ids.X.text) > 10):
                self.ids.X.text = "10.0"
            elif (float(self.ids.X.text) < -10):
                self.ids.X.text = "-10.0"

        except:
            pass

    def check_text_y(self):
        try:
            self.ids.Y.text = str(float(self.ids.Y.text))
            if (float(self.ids.Y.text) > 10):
                self.ids.Y.text = "10.0"
            elif (float(self.ids.Y.text) < -10):
                self.ids.Y.text = "-10.0"

        except:
            pass

    def check_text_z(self):
        try:
            self.ids.Z.text = str(float(self.ids.Z.text))
            if (float(self.ids.Z.text) > 10):
                self.ids.Z.text = "10.0"
            elif (float(self.ids.Z.text) < -10):
                self.ids.Z.text = "-10.0"

        except:
            pass

    def interpolation(self):
        global image_volume
        x = float(self.ids.X.text)
        y = float(self.ids.Y.text)
        z = float(self.ids.Z.text)
        image_volume = interpolate(image_volume, x, y, z)
        build_gif()

class FilterSelection(Popup):

    def median_filter(self):
        global image_volume
        filter_size = int(self.ids.filter_number.text)
        image_volume = median_filter_3d(image_volume, filter_size)
        build_gif()

    def min_filter(self):
        global image_volume
        filter_size = int(self.ids.filter_number.text)
        image_volume = min_filter_3d(image_volume, filter_size)
        build_gif()

    def max_filter(self):
        global image_volume
        filter_size = int(self.ids.filter_number.text)
        image_volume = max_filter_3d(image_volume, filter_size)
        build_gif()

    def check_text(self):
        try:
            self.ids.filter_number.text = str(int(self.ids.filter_number.text))
            if (int(self.ids.filter_number.text) > 100):
                self.ids.filter_number.text = "100"
            elif (int(self.ids.filter_number.text) < 0):
                self.ids.filter_number.text = "0"

        except:
            pass


class MorphologicalSelection(Popup):
    def small_objects(self):
        global image_list, image_volume
        image_volume = remove_small_obj(image_list)
        build_gif()

    def skeletonize(self):
        global image_volume
        image_volume = skeletonize(image_volume)
        build_gif()

    def erosion(self):
        global image_list, image_volume
        image_volume = erosion(image_list)
        build_gif()


class GifView(Popup):
    def on_pre_open(self):
        self.ids.resulted_gif.source = 'GIFs/result.gif'
        self.ids.resulted_gif.reload()

def build_gif():
    # Build GIF
    # gif_build('result', image_list)
    gif_build('result', image_volume)

# Create the screen manager
sm = ScreenManager()
sm.add_widget(StartingScreen(name='starting'))
# sm.add_widget(P2_screen)
# sm.add_widget(P3_screen)
# sm.add_widget(P4Screen(name='P4'))


class Demo(App):
    def on_start(self):
        try:
            os.remove("GIFs/result.gif")
        except:
            pass

    def build(self):
        return sm



if __name__ == '__main__':
    Demo().run()